import 'dart:async';

import 'package:flutter/services.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:vgk_game_services/games_services_platform_interface/helpers.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/achievement.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/score.dart';

const MethodChannel _channel = const MethodChannel("vgk_game_services");

class MethodChannelGamesServices extends PlatformInterface {
  MethodChannelGamesServices() : super(token: _token);
  static final Object _token = Object();
  static MethodChannelGamesServices _instance = MethodChannelGamesServices();

  static MethodChannelGamesServices get instance => _instance;

  static set instance(MethodChannelGamesServices instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<String?> unlock({achievement: Achievement}) async {
    return await _channel.invokeMethod("unlock", {
      "achievementID": achievement.id,
      "percentComplete": achievement.percentComplete,
    });
  }

  Future<String?> submitScore({score: Score}) async {
    return await _channel.invokeMethod("submitScore", {
      "leaderboardID": score.leaderboardID,
      "value": score.value,
    });
  }

  Future<String?> increment({achievement: Achievement}) async {
    return await _channel.invokeMethod("increment", {
      "achievementID": achievement.id,
      "steps": achievement.steps,
    });
  }

  Future<String?> showAchievements() async {
    return await _channel.invokeMethod("showAchievements");
  }

  Future<String?> showLeaderboards(
      {iOSLeaderboardID = "", androidLeaderboardID = ""}) async {
    return await _channel.invokeMethod("showLeaderboards", {
      "leaderboardID":
          Helpers.isPlatformAndroid ? androidLeaderboardID : iOSLeaderboardID
    });
  }

  Future<String?> signIn() async {
    if (Helpers.isPlatformAndroid) {
      return await _channel.invokeMethod("silentSignIn");
    } else {
      return await _channel.invokeMethod("signIn");
    }
  }

  Future<bool?> get isSignedIn => _channel.invokeMethod("isSignedIn");

  Future<String?> signOut() async {
    return await _channel.invokeMethod("signOut");
  }

  Future<String?> hideAccessPoint() async {
    return await _channel.invokeMethod("hideAccessPoint");
  }

  Future<String?> getPlayerID() async {
    return await _channel.invokeMethod("getPlayerID");
  }
}
